body {
  padding: 50px;
  font-family: sans-serif;
  font-size: 14px;
}

.test-lr {
  padding: 4em;
  color: #444;
  /* color: red; */
  font-weight: bolder;

  border: #ccc;
  background-color: #dedede;
}
